(* latex-entities.sml
 *
 * COPYRIGHT (c) 2000 Bell Labs, Lucent Technologies
 *
 * The translation for the standard ML-Doc entities to LaTeX.
 *)

structure EntitiesToLaTeX = EntitiesFn (
    val entityMap = [
	  ("IE",	"\\mldIE{}"),
	  ("EG",	"\\mldEG{}"),
	  ("LT",	"\\texttt{<}"),
	  ("LTE",	"\\texttt{<=}"),
	  ("GT",	"\\texttt{>}"),
	  ("GTE",	"\\texttt{>=}"),
	  ("NEQ",	"\\texttt{<>}"),
	  ("AMP",	"\\&"),
	  ("DQUOTE",	"\""),
	  ("BAR",	"\\texttt{|}"),	(* could be \ensuremath{\mid} *)
	  ("CUP",	"\\cup{}"),
	  ("CAP",	"\\cap{}"),
	  ("EXISTS",	"\\exists{}"),
	  ("FORALL",	"\\forall{}"),
	  ("GREATER",	">"),
	  ("GREATEREQ",	"\\geq{}"),
	  ("IN",	"\\in{}"),
	  ("LESS",	"<"),
	  ("LESSEQ",	"\\leq{}"),
	  ("NOTEQ",	"\\neq{}"),
	  ("NOTIN",	"\\not\\in{}"),
	  ("OMINUS",	"\\ominus{}"),
	  ("OPLUS",	"\\oplus{}"), 
	  ("OTIMES",	"\\otimes{}"),
          ("PI",        "\\pi{}"), 
          ("PLUSMINUS", "\\pm{}"),
          ("MINUSPLUS", "\\mp{}"),
          ("INF",       "\\infty{}"),

	(* the HTML 2.0 entities *)
	  ("nbsp",	"~"),
	  ("iexcl",	"&iexcl;"),
	  ("cent",	"&cent;"),
	  ("pound",	"&pound;"),
	  ("curren",	"&curren;"),
	  ("yen",	"&yen;"),
	  ("brvbar",	"&brvbar;"),
	  ("sect",	"&sect;"),
	  ("uml",	"&uml;"),
	  ("copy",	"&copy;"),
	  ("ordf",	"&ordf;"),
	  ("laquo",	"&laquo;"),
	  ("not",	"&not;"),
	  ("shy",	"&shy;"),
	  ("reg",	"&reg;"),
	  ("macr",	"&macr;"),
	  ("deg",	"&deg;"),
	  ("plusmn",	"&plusmn;"),
	  ("sup2",	"&sup2;"),
	  ("sup3",	"&sup3;"),
	  ("acute",	"&acute;"),
	  ("micro",	"&micro;"),
	  ("para",	"&para;"),
	  ("middot",	"&middot;"),
	  ("cedil",	"&cedil;"),
	  ("sup1",	"&sup1;"),
	  ("ordm",	"&ordm;"),
	  ("raquo",	"&raquo;"),
	  ("frac14",	"&frac14;"),
	  ("frac12",	"&frac12;"),
	  ("frac34",	"&frac34;"),
	  ("iquest",	"&iquest;"),
	  ("Agrave",	"&Agrave;"),
	  ("Aacute",	"&Aacute;"),
	  ("Acirc",	"&Acirc;"),
	  ("Atilde",	"&Atilde;"),
	  ("Auml",	"&Auml;"),
	  ("Aring",	"&Aring;"),
	  ("AElig",	"&AElig;"),
	  ("Ccedil",	"&Ccedil;"),
	  ("Egrave",	"&Egrave;"),
	  ("Eacute",	"&Eacute;"),
	  ("Ecirc",	"&Ecirc;"),
	  ("Euml",	"&Euml;"),
	  ("Igrave",	"&Igrave;"),
	  ("Iacute",	"&Iacute;"),
	  ("Icirc",	"&Icirc;"),
	  ("Iuml",	"&Iuml;"),
	  ("ETH",	"&ETH;"),
	  ("Ntilde",	"&Ntilde;"),
	  ("Ograve",	"&Ograve;"),
	  ("Oacute",	"&Oacute;"),
	  ("Ocirc",	"&Ocirc;"),
	  ("Otilde",	"&Otilde;"),
	  ("Ouml",	"&Ouml;"),
	  ("times",	"&times;"),
	  ("Oslash",	"&Oslash;"),
	  ("Ugrave",	"&Ugrave;"),
	  ("Uacute",	"&Uacute;"),
	  ("Ucirc",	"&Ucirc;"),
	  ("Uuml",	"\\\"{U}"),
	  ("Yacute",	"&Yacute;"),
	  ("THORN",	"&THORN;"),
	  ("szlig",	"&szlig;"),
	  ("agrave",	"&agrave;"),
	  ("aacute",	"&aacute;"),
	  ("acirc",	"&acirc;"),
	  ("atilde",	"&atilde;"),
	  ("auml",	"&auml;"),
	  ("aring",	"&aring;"),
	  ("aelig",	"&aelig;"),
	  ("ccedil",	"&ccedil;"),
	  ("egrave",	"&egrave;"),
	  ("eacute",	"&eacute;"),
	  ("ecirc",	"&ecirc;"),
	  ("euml",	"&euml;"),
	  ("igrave",	"&igrave;"),
	  ("iacute",	"&iacute;"),
	  ("icirc",	"&icirc;"),
	  ("iuml",	"&iuml;"),
	  ("eth",	"&eth;"),
	  ("ntilde",	"&ntilde;"),
	  ("ograve",	"&ograve;"),
	  ("oacute",	"&oacute;"),
	  ("ocirc",	"&ocirc;"),
	  ("otilde",	"&otilde;"),
	  ("ouml",	"&ouml;"),
	  ("divide",	"&divide;"),
	  ("oslash",	"&oslash;"),
	  ("ugrave",	"&ugrave;"),
	  ("uacute",	"&uacute;"),
	  ("ucirc",	"&ucirc;"),
	  ("uuml",	"\\\"{u}"),
	  ("yacute",	"&yacute;"),
	  ("thorn",	"&thorn;"),
	  ("yuml",	"&yuml;")
	]
  );


structure EntitiesToCodeLaTeX = EntitiesFn (
    val entityMap = [
	  ("LT",	"<"),
	  ("LTE",	"<="),
	  ("GT",	">"),
	  ("GTE",	">="),
	  ("NEQ",	"<>"),
	  ("AMP",	"&"),
	  ("DQUOTE",	"\""),
	  ("BAR",	"|"),	(* could be \ensuremath{\mid} *)
	  ("CUP",	"\\ensuremath{\\cup{}}"),
	  ("CAP",	"\\ensuremath{\\cap{}}"),
	  ("EXISTS",	"\\ensuremath{\\exists{}}"),
	  ("FORALL",	"\\ensuremath{\\forall{}}"),
	  ("GREATER",	"\\ensuremath{>}"),
	  ("GREATEREQ",	"\\ensuremath{\\geq{}}"),
	  ("IN",	"\\ensuremath{\\in{}}"),
	  ("LESS",	"\\ensuremath{<}"),
	  ("LESSEQ",	"\\ensuremath{\\leq{}}"),
	  ("NOTEQ",	"\\ensuremath{\\neq{}}"),
	  ("NOTIN",	"\\ensuremath{\\not\\in{}}"),
	  ("OMINUS",	"\\ensuremath{\\ominus{}}"),
	  ("OPLUS",	"\\ensuremath{\\oplus{}}"), 
	  ("OTIMES",	"\\ensuremath{\\otimes{}}"),
          ("PI",        "\\ensuremath{\\pi{}}"), 
          ("PLUSMINUS", "\\ensuremath{\\pm{}}"),
          ("MINUSPLUS", "\\ensuremath{\\mp{}}"),
          ("INF",       "\\ensuremath{\\infty{}}"),

	(* the HTML 2.0 entities *)
	  ("nbsp",	" ")
	]
  );
