
structure PPStrm = PPStreamFn (
  structure Token = StringToken
  structure Device = SimpleTextIODev);

(* DEBUG structure PPStrm = PPDebugFn (PPStrm);*)

structure Style =
  struct
    type token = StringToken.token
    type style = StringToken.style
    type context = unit

    datatype xref_kind
      = SigRef | StrRef | FctRef | ShareRef | TyRef | ExnRef | ConRef | ValRef

    val kwStyle = ()
    val punctStyle = ()
    val tyvarStyle = ()
    val idStyle = ()

  (* identifiers that are possible cross references *)
    fun idToToken () (MLSpec.ID id) = id

  (* identifiers that are forward links to descriptions *)
    fun descRef () (_, id) = id

  end

structure PP = PPSpecFn(
  structure PPStrm = PPStrm
  structure Style = Style);

structure Main = struct
local
fun mkQId m = let
      fun mkId [] = []
	| mkId [id] = [id]
	| mkId (id::r) = id :: "." :: mkId r
      in
	MLSpec.ID(String.concat(mkId (MLDocType.scanTyId m)))
      end
fun parseTy s = MLDocType.parseTy mkQId [MLDocMarkup.DATA s]
in
fun ppTy ty = let
      val ppStrm =
	    PPStrm.openStream(
	      SimpleTextIODev.openDev {dst = TextIO.stdOut, wid = 40})
      in
	print(concat["Type: \"", ty, "\"\n"]);
	print "Result:\n";
	PP.ppType ((), ppStrm) (parseTy ty);
	PPStrm.closeStream ppStrm;
	print "\n"
      end;
fun ppValSpecs specs = let
      val ppStrm =
	    PPStrm.openStream(
	      SimpleTextIODev.openDev {dst = TextIO.stdOut, wid = 40})
      fun cvt (id, ty) = (id, parseTy ty)
      in
	PP.ppValSpec ((), ppStrm, true) (map cvt specs);
	PPStrm.closeStream ppStrm
      end
end (* local *)
end;

